@@ -1,25 +1,12 @@
 /*
-    This file is part of kdev-python, the python language plugin for KDevelop
-    Copyright (C) 2012  Sven Brauch <svenbrauch@googlemail.com>
+    SPDX-FileCopyrightText: 2012 Sven Brauch <svenbrauch@googlemail.com>
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-
 #include "pdbframestackmodel.h"
 #include "debugsession.h"
-#include <QRegExp>
+#include <QRegularExpression>
 
 #include <QDebug>
 #include "debuggerdebug.h"
@@ -47,13 +34,13 @@ void PdbFrameStackModel::setDebuggerAtFrame(int newFrame)
 void PdbFrameStackModel::framesFetched(QByteArray framelist)
 {
     qCDebug(KDEV_PYTHON_DEBUGGER) << "frames fetched:" << framelist;
-    QList<QByteArray> lines = framelist.split('\n');
+    QStringList lines = QString::fromLatin1(framelist).split(QLatin1Char('\n'));
     QList<FrameItem> frames;
     bool parsingLocation = false;
     FrameItem* currentFrame = nullptr;
     int framesCount = 0;
-    foreach ( const QString& line, lines ) {
-        if ( line.startsWith("-> ") ) {
+    for (const QString& line : std::as_const(lines)) {
+        if ( line.startsWith(QStringLiteral("-> ")) ) {
             parsingLocation = true;
             if ( currentFrame ) {
                 frames << *currentFrame;
@@ -63,17 +50,18 @@ void PdbFrameStackModel::framesFetched(QByteArray framelist)
             framesCount++;
         }
         else if ( parsingLocation ) {
-            QRegExp location("(\\>?)\\s*(.*)\\(([0-9]+)\\)(.*)");
+            static QRegularExpression location(QStringLiteral("(\\>?)\\s*(.*)\\(([0-9]+)\\)(.*)"));
             // version 1 has some *really* weird "greedy" ruleset which makes no sense at all for me
-            location.setPatternSyntax(QRegExp::RegExp2);
-            if ( location.exactMatch(line) ) {
-                qCDebug(KDEV_PYTHON_DEBUGGER) << location.capturedTexts();
-                if ( ! location.capturedTexts().at(1).isEmpty() ) {
+            //location.setPatternSyntax(QRegExp::RegExp2);
+            const auto match = location.match(line);
+            if ( match.hasMatch() ) {
+                qCDebug(KDEV_PYTHON_DEBUGGER) << match.capturedView();
+                if ( ! match.captured(1).isEmpty() ) {
                     m_debuggerAtFrame = framesCount;
                 }
-                currentFrame->file = QUrl::fromLocalFile(location.capturedTexts().at(2));
-                currentFrame->line = location.capturedTexts().at(3).toInt() - 1;
-                currentFrame->name = location.capturedTexts().at(4);
+                currentFrame->file = QUrl::fromLocalFile(match.captured(2));
+                currentFrame->line = match.captured(3).toInt() - 1;
+                currentFrame->name = match.captured(4);
             }
             else {
                 qCDebug(KDEV_PYTHON_DEBUGGER) << "regular expression mismatches" << line;
@@ -98,7 +86,7 @@ void PdbFrameStackModel::threadsFetched(QByteArray threadsData)
     QVector<ThreadItem> threads;
     ThreadItem mainThread;
     mainThread.nr = 0;
-    mainThread.name = "main thread";
+    mainThread.name = QStringLiteral("main thread");
     threads << mainThread;
     setThreads(threads);
     setCurrentThread(0);
@@ -107,7 +95,7 @@ void PdbFrameStackModel::threadsFetched(QByteArray threadsData)
 void PdbFrameStackModel::fetchFrames(int /*threadNumber*/, int /*from*/, int /*to*/)
 {
     qCDebug(KDEV_PYTHON_DEBUGGER) << "frames requested";
-    InternalPdbCommand* cmd = new InternalPdbCommand(this, "framesFetched", "where\n");
+    InternalPdbCommand* cmd = new InternalPdbCommand(this, "framesFetched", QStringLiteral("where\n"));
     static_cast<DebugSession*>(session())->addCommand(cmd);
 }
 
@@ -115,9 +103,10 @@ void PdbFrameStackModel::fetchThreads()
 {
     qCDebug(KDEV_PYTHON_DEBUGGER) << "threads requested";
     // pdb doesn't support threads.
-    InternalPdbCommand* cmd = new InternalPdbCommand(this, "threadsFetched", "pass\n");
+    InternalPdbCommand* cmd = new InternalPdbCommand(this, "threadsFetched", QStringLiteral("pass\n"));
     static_cast<DebugSession*>(session())->addCommand(cmd);
 }
 
 }
 
+#include "moc_pdbframestackmodel.cpp"
