@@ -58,7 +58,7 @@ DeclarationBuilder:: ~DeclarationBuilder()
 {
     if ( ! m_scheduledForDeletion.isEmpty() ) {
         DUChainWriteLocker lock;
-        foreach ( DUChainBase* d, m_scheduledForDeletion ) {
+        for (DUChainBase* d : std::as_const(m_scheduledForDeletion)) {
             delete d;
         }
         m_scheduledForDeletion.clear();
@@ -177,7 +177,12 @@ template<typename T> QList<Declaration*> DeclarationBuilder::reopenFittingDeclar
     // Search for a declaration from a previous parse pass which should be re-used
     QList<Declaration*> remainingDeclarations;
     *ok = nullptr;
-    foreach ( Declaration* d, declarations ) {
+    for ( Declaration* d : declarations ) {
+        if ( ! d ) {
+            qCWarning(KDEV_PYTHON_DUCHAIN) << "null declaration in candidate list";
+            continue;
+        }
+
         Declaration* fitting = dynamic_cast<T*>(d);
         if ( ! fitting ) {
             // Only use a declaration if the type matches
@@ -353,7 +358,7 @@ void DeclarationBuilder::visitWithItem(WithItemAst* node)
         auto mgrType = v.lastType();
         auto enterType = mgrType; // If we can't find __enter__(), assume it returns `self` like file objects.
 
-        static const IndexedIdentifier enterId(KDevelop::Identifier("__enter__"));
+        static const IndexedIdentifier enterId(KDevelop::Identifier(QStringLiteral("__enter__")));
 
         DUChainReadLocker lock;
         if ( auto enterFunc = dynamic_cast<FunctionDeclaration*>(
@@ -387,7 +392,7 @@ Declaration* DeclarationBuilder::findDeclarationInContext(QStringList dottedName
     Declaration* lastAccessedDeclaration = nullptr;
     int i = 0;
     int identifierCount = dottedNameIdentifier.length();
-    foreach ( const QString& currentIdentifier, dottedNameIdentifier ) {
+    for ( const QString& currentIdentifier : dottedNameIdentifier ) {
         Q_ASSERT(currentContext);
         i++;
         QList<Declaration*> declarations = currentContext->findDeclarations(QualifiedIdentifier(currentIdentifier).first(),
@@ -410,14 +415,14 @@ QString DeclarationBuilder::buildModuleNameFromNode(ImportFromAst* node, AliasAs
 {
     QString moduleName = alias->name->value;
     if ( ! intermediate.isEmpty() ) {
-        moduleName.prepend('.').prepend(intermediate);
+        moduleName.prepend(QLatin1Char('.')).prepend(intermediate);
     }
     if ( node->module ) {
-        moduleName.prepend('.').prepend(node->module->value);
+        moduleName.prepend(QLatin1Char('.')).prepend(node->module->value);
     }
     // To handle relative imports correctly, add node level in the beginning of the path
     // This will allow findModulePath to deduce module search direcotry properly
-    moduleName.prepend(QString(node->level, '.'));
+    moduleName.prepend(QString(node->level, QLatin1Char('.')));
     return moduleName;
 }
 
@@ -426,7 +431,7 @@ void DeclarationBuilder::visitImportFrom(ImportFromAst* node)
     Python::AstDefaultVisitor::visitImportFrom(node);
     QString moduleName;
     QString declarationName;
-    foreach ( AliasAst* name, node->names ) {
+    for (AliasAst* name : std::as_const(node->names)) {
         // iterate over all the names that are imported, like "from foo import bar as baz, bang as asdf"
         Identifier* declarationIdentifier = nullptr;
         declarationName.clear();
@@ -448,7 +453,7 @@ void DeclarationBuilder::visitImportFrom(ImportFromAst* node)
         Declaration* success = createModuleImportDeclaration(moduleName, declarationName, declarationIdentifier, problem);
         if ( ! success && (node->module || node->level) ) {
             ProblemPointer problem_init(nullptr);
-            intermediate = QString("__init__");
+            intermediate = QStringLiteral("__init__");
             moduleName = buildModuleNameFromNode(node, name, intermediate);
             success = createModuleImportDeclaration(moduleName, declarationName, declarationIdentifier, problem_init);
         }
@@ -471,7 +476,7 @@ void DeclarationBuilder::visitImport(ImportAst* node)
 {
     Python::ContextBuilder::visitImport(node);
     DUChainWriteLocker lock;
-    foreach ( AliasAst* name, node->names ) {
+    for (AliasAst* name : std::as_const(node->names)) {
         QString moduleName = name->name->value;
         // use alias if available, name otherwise
         Identifier* declarationIdentifier = name->asName ? name->asName : name->name;
@@ -619,7 +624,8 @@ Declaration* DeclarationBuilder::createDeclarationTree(const QStringList& nameCo
         auto moduleContext = openContext(declarationIdentifier, KDevelop::DUContext::Other, &contextIdentifier);
         openedContexts.append(moduleContext);
 
-        foreach ( Declaration* local, currentContext()->localDeclarations() ) {
+        const auto localDeclarations = currentContext()->localDeclarations();
+        for (Declaration* local : localDeclarations) {
             // keep all the declarations until the builder finished
             // kdevelop would otherwise delete them as soon as the context is closed
             if ( ! wasEncountered(local) ) {
@@ -712,22 +718,22 @@ Declaration* DeclarationBuilder::createModuleImportDeclaration(QString moduleNam
     }
     if ( moduleInfo.second.isEmpty() ) {
         // import the whole module
-        resultingDeclaration = createDeclarationTree(declarationName.split("."),
+        resultingDeclaration = createDeclarationTree(declarationName.split(QLatin1Char('.')),
                                                      declarationIdentifier, moduleContext, nullptr, range);
         auto initFile = QStringLiteral("/__init__.py");
         auto path = moduleInfo.first.path();
         if ( path.endsWith(initFile) ) {
             // if the __init__ file is imported, import all the other files in that directory as well
             QDir dir(path.left(path.size() - initFile.size()));
-            dir.setNameFilters({"*.py"});
+            dir.setNameFilters({QStringLiteral("*.py")});
             dir.setFilter(QDir::Files);
-            auto files = dir.entryList();
-            foreach ( const auto& file, files ) {
+            const auto files = dir.entryList();
+            for ( const auto& file : files ) {
                 if ( file == QStringLiteral("__init__.py") ) {
                     continue;
                 }
-                const auto filePath = declarationName.split(".") << file.left(file.lastIndexOf(".py"));
-                const auto fileUrl = QUrl::fromLocalFile(dir.path() + "/" + file);
+                const auto filePath = declarationName.split(QLatin1Char('.')) << file.left(file.lastIndexOf(QStringLiteral(".py")));
+                const auto fileUrl = QUrl::fromLocalFile(dir.path() + QLatin1Char('/') + file);
                 ReferencedTopDUContext fileContext;
                 {
                     DUChainReadLocker lock;
@@ -735,7 +741,7 @@ Declaration* DeclarationBuilder::createModuleImportDeclaration(QString moduleNam
                 }
                 if ( fileContext ) {
                     Identifier id = *declarationIdentifier;
-                    id.value.append(".").append(filePath.last());
+                    id.value.append(QLatin1Char('.')).append(filePath.last());
                     createDeclarationTree(filePath,
                                           &id, fileContext, nullptr);
                 }
@@ -749,7 +755,7 @@ Declaration* DeclarationBuilder::createModuleImportDeclaration(QString moduleNam
     else {
         // import a specific declaration from the given file
         lock.lock();
-        if ( declarationIdentifier->value == "*" ) {
+        if ( declarationIdentifier->value == QLatin1Char('*') ) {
             qCDebug(KDEV_PYTHON_DUCHAIN) << "Importing * from module";
             currentContext()->addImportedParentContext(moduleContext);
         }
@@ -758,7 +764,7 @@ Declaration* DeclarationBuilder::createModuleImportDeclaration(QString moduleNam
             Declaration* originalDeclaration = findDeclarationInContext(moduleInfo.second, moduleContext);
             if ( originalDeclaration ) {
                 DUChainWriteLocker lock(DUChain::lock());
-                resultingDeclaration = createDeclarationTree(declarationName.split("."), declarationIdentifier,
+                resultingDeclaration = createDeclarationTree(declarationName.split(QLatin1Char('.')), declarationIdentifier,
                                                              ReferencedTopDUContext(nullptr), originalDeclaration,
                                                              editorFindRange(declarationIdentifier, declarationIdentifier));
             }
@@ -767,7 +773,7 @@ Declaration* DeclarationBuilder::createModuleImportDeclaration(QString moduleNam
                 p->setFinalLocation(DocumentRange(currentlyParsedDocument(), range.castToSimpleRange())); // TODO ok?
                 p->setSource(KDevelop::IProblem::SemanticAnalysis);
                 p->setSeverity(KDevelop::IProblem::Warning);
-                p->setDescription(i18n("Declaration for \"%1\" not found in specified module", moduleInfo.second.join(".")));
+                p->setDescription(i18n("Declaration for \"%1\" not found in specified module", moduleInfo.second.join(QLatin1Char('.'))));
                 problemEncountered = p;
             }
         }
@@ -806,7 +812,7 @@ void DeclarationBuilder::visitYield(YieldAst* node)
     else {
         // Otherwise, create a new container type, and set it as the function's return type.
         DUChainWriteLocker lock;
-        auto container = ExpressionVisitor::typeObjectForIntegralType<ListType>("list");
+        auto container = ExpressionVisitor::typeObjectForIntegralType<ListType>(QStringLiteral("list"));
         if ( container ) {
             openType(container);
             container->addContentType<Python::UnsureType>(encountered);
@@ -822,7 +828,7 @@ void DeclarationBuilder::visitLambda(LambdaAst* node)
     // A context must be opened, because the lamdba's arguments are local to the lambda:
     // d = lambda x: x*2; print x # <- gives an error
     openContext(node, editorFindRange(node, node->body), DUContext::Other);
-    foreach ( ArgAst* argument, node->arguments->arguments ) {
+    for (ArgAst* argument : std::as_const(node->arguments->arguments)) {
         visitVariableDeclaration<Declaration>(argument->argumentName);
     }
     visitNodeList(node->arguments->defaultValues);
@@ -853,7 +859,7 @@ void DeclarationBuilder::applyDocstringHints(CallAst* node, FunctionDeclaration:
     // Check for the different types of modifiers such a function can have
     QStringList args;
     QHash< QString, std::function<void()> > items;
-    items["addsTypeOfArg"] = [&]() {
+    items[QStringLiteral("addsTypeOfArg")] = [&]() {
         const int offset = ! args.isEmpty() ? (int) args.at(0).toUInt() : 0;
         if ( node->arguments.length() <= offset ) {
             return;
@@ -870,7 +876,7 @@ void DeclarationBuilder::applyDocstringHints(CallAst* node, FunctionDeclaration:
         container->addContentType<Python::UnsureType>(argVisitor.lastType());
         v.lastDeclaration()->setType(container);
     };
-    items["addsTypeOfArgContent"] = [&]() {
+    items[QStringLiteral("addsTypeOfArgContent")] = [&]() {
         const int offset = ! args.isEmpty() ? (int) args.at(0).toUInt() : 0;
         if ( node->arguments.length() <= offset ) {
             return;
@@ -884,11 +890,11 @@ void DeclarationBuilder::applyDocstringHints(CallAst* node, FunctionDeclaration:
             v.lastDeclaration()->setType(container);
         }
     };
-    auto docstring = function->comment();
+    auto docstring = QString::fromLatin1(function->comment());
     if ( ! docstring.isEmpty() ) {
-        foreach ( const auto& key, items.keys() ) {
-            if ( Helper::docstringContainsHint(docstring, key, &args) ) {
-                items[key]();
+        for (auto i = items.cbegin(), end = items.cend(); i != end; ++i) {
+            if (Helper::docstringContainsHint(docstring, i.key(), &args)) {
+                items[i.key()]();
             }
         }
     }
@@ -1012,7 +1018,7 @@ void DeclarationBuilder::addArgumentTypeHints(CallAst* node, DeclarationPointer
     }
     lock.unlock();
     DUChainWriteLocker wlock;
-    foreach ( KeywordAst* keyword, node->keywords ) {
+    for (KeywordAst* keyword : std::as_const(node->keywords)) {
         wlock.unlock();
         ExpressionVisitor argumentVisitor(currentContext());
         argumentVisitor.visitNode(keyword->value);
@@ -1063,7 +1069,7 @@ void DeclarationBuilder::visitMatch(MatchAst* node)
     ExpressionVisitor subjectVisitor(currentContext());
     subjectVisitor.visitNode(node->subject);
 
-    for (auto* matchCase: node->cases) {
+    for (auto* matchCase : std::as_const(node->cases)) {
         if (!matchCase || !matchCase->pattern) {
             continue;
         }
@@ -1072,7 +1078,7 @@ void DeclarationBuilder::visitMatch(MatchAst* node)
         switch (matchCase->pattern->astType) {
             case Ast::MatchSequenceAstType: {
                 auto* seq = static_cast<MatchSequenceAst*>(matchCase->pattern);
-                for (auto* element: seq->patterns) {
+                for (auto* element : std::as_const(seq->patterns)) {
                     if (element->astType != Ast::MatchAsAstType) {
                         continue;
                     }
@@ -1315,7 +1321,7 @@ void DeclarationBuilder::assignToTuple(TupleAst* tuple, const SourceType& elemen
         auto target = tuple->elements.at(ii);
         if ( target->astType == Ast::StarredAstType ) {
             DUChainReadLocker lock;
-            auto listType = ExpressionVisitor::typeObjectForIntegralType<ListType>("list");
+            auto listType = ExpressionVisitor::typeObjectForIntegralType<ListType>(QStringLiteral("list"));
             lock.unlock();
             if (listType) {
                 listType->addContentType<Python::UnsureType>(sourceType);
@@ -1368,7 +1374,7 @@ void DeclarationBuilder::visitAssignment(AssignmentAst* node)
         v.isAlias()
     };
 
-    foreach(ExpressionAst* target, node->targets) {
+    for (ExpressionAst* target : std::as_const(node->targets)) {
         assignToUnknown(target, sourceType);
     }
 }
@@ -1413,9 +1419,9 @@ void DeclarationBuilder::visitClassDefinition( ClassDefinitionAst* node )
     dec->setComment(docstring);
     if ( ! docstring.isEmpty() ) {
         // check whether this is a type container (list, dict, ...) or just a "normal" class
-        if ( Helper::docstringContainsHint(docstring, "TypeContainer") ) {
+        if ( Helper::docstringContainsHint(docstring, QStringLiteral("TypeContainer")) ) {
             ListType* container = nullptr;
-            if ( Helper::docstringContainsHint(docstring, "hasTypedKeys") ) {
+            if ( Helper::docstringContainsHint(docstring, QStringLiteral("hasTypedKeys")) ) {
                 container = new MapType();
             }
             else {
@@ -1423,13 +1429,13 @@ void DeclarationBuilder::visitClassDefinition( ClassDefinitionAst* node )
             }
             type = StructureType::Ptr(container);
         }
-        if ( Helper::docstringContainsHint(docstring, "IndexedTypeContainer") ) {
+        if ( Helper::docstringContainsHint(docstring, QStringLiteral("IndexedTypeContainer")) ) {
             IndexedContainer* container = new IndexedContainer();
             type = StructureType::Ptr(container);
         }
     }
     lock.unlock();
-    foreach ( ExpressionAst* c, node->baseClasses ) {
+    for (ExpressionAst* c : std::as_const(node->baseClasses)) {
         // Iterate over all the base classes, and add them to the duchain.
         ExpressionVisitor v(currentContext());
         v.visitNode(c);
@@ -1446,12 +1452,12 @@ void DeclarationBuilder::visitClassDefinition( ClassDefinitionAst* node )
     lock.lock();
     // every python class inherits from "object".
     // We use this to add all the __str__, __get__, ... methods.
-    if ( dec->baseClassesSize() == 0 && node->name->value != "object" ) {
+    if ( dec->baseClassesSize() == 0 && node->name->value != QStringLiteral("object") ) {
         DUChainWriteLocker wlock;
         ReferencedTopDUContext docContext = Helper::getDocumentationFileContext();
         if ( docContext ) {
             QList<Declaration*> object = docContext->findDeclarations(
-                QualifiedIdentifier("object")
+                QualifiedIdentifier(QStringLiteral("object"))
             );
             if ( ! object.isEmpty() && object.first()->abstractType() ) {
                 Declaration* objDecl = object.first();
@@ -1511,7 +1517,7 @@ void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
     dec->setStatic(false);
     dec->setClassMethod(false);
     dec->setProperty(false);
-    foreach ( auto decorator, node->decorators) {
+    for (auto decorator : std::as_const(node->decorators)) {
         visitNode(decorator);
         switch (decorator->astType) {
           case Ast::AttributeAstType: {
@@ -1572,15 +1578,11 @@ void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
             static IndexedString clsArgumentName("cls");
             if ( currentContext()->type() == DUContext::Class && ! parameters.isEmpty() && ! dec->isClassMethod() ) {
                 QString description;
-                if ( dec->identifier().identifier() == newMethodName
-                     && parameters[0]->identifier().identifier() != clsArgumentName )
+                if ( parameters[0]->identifier().identifier() != clsArgumentName
+                     && parameters[0]->identifier().identifier() != selfArgumentName )
                 {
-                    description = i18n("First argument of __new__ method is not called cls, this is deprecated");
-                }
-                else if ( dec->identifier().identifier() != newMethodName
-                          && parameters[0]->identifier().identifier() != selfArgumentName )
-                {
-                    description = i18n("First argument of class method is not called self, this is deprecated");
+                    // https://peps.python.org/pep-0008/#function-and-method-arguments
+                    description = i18n("First argument of class method is not called self or cls, this is deprecated");
                 }
                 if ( ! description.isEmpty() ) {
                     DUChainWriteLocker lock;
@@ -1790,7 +1792,8 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
     int totalArgCount = parametersCount + posonlyCount + kwonlyCount;
     int firstDefaultKwParameterOffset = totalArgCount - defaultKwParametersCount;
     int currentIndex = 0;
-    foreach ( ArgAst* arg, node->posonlyargs + node->arguments + node->kwonlyargs ) {
+    for (ArgAst* arg :
+         std::as_const(node->posonlyargs) + std::as_const(node->arguments) + std::as_const(node->kwonlyargs)) {
         // Iterate over all the function's arguments, create declarations, and add the arguments
         // to the functions FunctionType.
         currentIndex += 1;
@@ -1871,7 +1874,7 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
         // inject the vararg at the correct place
         int atIndex = 0;
         int useIndex = -1;
-        foreach ( ArgAst* arg, node->arguments ) {
+        for (ArgAst* arg : std::as_const(node->arguments)) {
             if ( node->vararg && workingOnDeclaration->vararg() == -1 && node->vararg->appearsBefore(arg) ) {
                 useIndex = atIndex;
             }
@@ -1883,7 +1886,7 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
             useIndex = type->arguments().size();
         }
         DUChainReadLocker lock;
-        IndexedContainer::Ptr tupleType = ExpressionVisitor::typeObjectForIntegralType<IndexedContainer>("tuple");
+        IndexedContainer::Ptr tupleType = ExpressionVisitor::typeObjectForIntegralType<IndexedContainer>(QStringLiteral("tuple"));
         lock.unlock();
         if ( tupleType ) {
             visitVariableDeclaration<Declaration>(node->vararg->argumentName, nullptr, tupleType);
@@ -1894,8 +1897,8 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
 
     if ( node->kwarg ) {
         DUChainReadLocker lock;
-        AbstractType::Ptr stringType = ExpressionVisitor::typeObjectForIntegralType<AbstractType>("str");
-        auto dictType = ExpressionVisitor::typeObjectForIntegralType<MapType>("dict");
+        AbstractType::Ptr stringType = ExpressionVisitor::typeObjectForIntegralType<AbstractType>(QStringLiteral("str"));
+        auto dictType = ExpressionVisitor::typeObjectForIntegralType<MapType>(QStringLiteral("dict"));
         lock.unlock();
         if ( dictType && stringType ) {
             dictType->addKeyType<Python::UnsureType>(stringType);
@@ -1923,7 +1926,7 @@ void DeclarationBuilder::visitNode(Ast* node) {
 void DeclarationBuilder::visitGlobal(GlobalAst* node)
 {
     TopDUContext* top = topContext();
-    foreach ( Identifier *id, node->names ) {
+    for (Identifier* id : std::as_const(node->names)) {
         QualifiedIdentifier qid = identifierForNode(id);
         DUChainWriteLocker lock;
         QList< Declaration* > existing = top->findLocalDeclarations(qid.first());
