@@ -1,20 +1,8 @@
-/**
-    This file is part of KDevelop
-    Copyright (C) 2011 Sven Brauch <svenbrauch@googlemail.com>
+/*
+    SPDX-FileCopyrightText: 2011 Sven Brauch <svenbrauch@googlemail.com>
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-**/
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "unsuretype.h"
 #include "helpers.h"
@@ -59,7 +47,7 @@ const QList<AbstractType::Ptr> UnsureType::typesRecursive() const
         AbstractType::Ptr current = type.abstractType();
         AbstractType::Ptr resolved = Helper::resolveAliasType(current);
         if ( resolved->whichType() == AbstractType::TypeUnsure ) {
-            results.append(resolved.cast<UnsureType>()->typesRecursive());
+            results.append(resolved.staticCast<UnsureType>()->typesRecursive());
         }
         else
             results.append(current);
@@ -72,7 +60,7 @@ QString UnsureType::toString() const
     QString typeList;
     QVector<AbstractType::Ptr> types;
     auto is_new_type = [&types](const IndexedType newType) {
-        foreach ( const auto& type, types ) {
+        for ( const auto& type : types ) {
             if ( type->indexed() == newType ) {
                 return false;
             }
@@ -80,7 +68,7 @@ QString UnsureType::toString() const
         return true;
     };
 
-    foreach ( AbstractType::Ptr type, typesRecursive() ) {
+    for ( AbstractType::Ptr type : typesRecursive() ) {
         if ( ! type ) {
             qCWarning(KDEV_PYTHON_DUCHAIN) << "Invalid type: " << type.data();
             continue;
@@ -112,7 +100,7 @@ QString UnsureType::toString() const
             // TODO collapse arguments / return type
             collapsedTypes.append(i18nc("some object that can be called, in programming", "<callable>"));
         }
-        auto have_iterable = count_and_remove([](T t) { return t.cast<IndexedContainer>() || t.cast<ListType>(); });
+        auto have_iterable = count_and_remove([](T t) { return t.dynamicCast<IndexedContainer>() || t.dynamicCast<ListType>(); });
         if ( have_iterable ) {
             // TODO collapse element count / types
             collapsedTypes.append(i18nc("a set with some elements", "<iterable>"));
@@ -120,16 +108,16 @@ QString UnsureType::toString() const
     }
 
     int count = 0;
-    foreach ( const auto& type, types ) {
+    for ( const auto& type : types ) {
         if ( count )
-            typeList += ", ";
+            typeList += QStringLiteral(", ");
         count += 1;
 
         typeList += type->toString();
     }
-    foreach ( const auto& collapsed, collapsedTypes ) {
+    for ( const auto& collapsed : collapsedTypes ) {
         if ( count )
-            typeList += ", ";
+            typeList += QStringLiteral(", ");
         count += 1;
 
         typeList += collapsed;
@@ -153,9 +141,9 @@ AbstractType::WhichType UnsureType::whichType() const
     return AbstractType::TypeUnsure;
 }
 
-void UnsureType::addType(IndexedType indexed) {
+void UnsureType::addType(const IndexedType& indexed) {
     auto type = indexed.abstractType();
-    auto hinted = type.cast<HintedType>(); // XXX: do we need a read locker here?
+    auto hinted = type.dynamicCast<HintedType>(); // XXX: do we need a read locker here?
     if ( ! hinted ) {
         // if we aren't adding a HintedType the default implementation works
         KDevelop::UnsureType::addType(indexed);
@@ -179,7 +167,7 @@ void UnsureType::addType(IndexedType indexed) {
             alreadyExists = true;
         }
         const auto& old = oldIndexed.abstractType();
-        if ( auto oldHinted = old.cast<HintedType>() ) {
+        if ( auto oldHinted = old.dynamicCast<HintedType>() ) {
             if ( !alreadyExists ) {
                 // only do these checks if we haven't already determined that it is a duplicate
                 auto oldHintedTarget = oldHinted->type()->indexed();
@@ -212,12 +200,12 @@ void UnsureType::addType(IndexedType indexed) {
             auto t = type2.abstractType();
             auto str = t->toString();
             types += "\n    " + QString::number(type2.index());
-            auto hinted = t.cast<HintedType>();
+            auto hinted = t.dynamicCast<HintedType>();
             while (hinted) {
                 auto target = hinted->type();
                 types += " (aka " + QString::number(target->indexed().index()) + ": " + target->toString()
                         +  " and context " + QString::number(hinted->createdBy().index()) + ")";
-                hinted = target.cast<HintedType>();
+                hinted = target.dynamicCast<HintedType>();
             }
             types += " - " + t->toString() + " of type "  + typeid(*t).name();
             if (!foundDuplicates) {
